#ifndef WINDOWS_H
#define WINDOWS_H

#ifdef __APPLE__
#include <libkern/OSTypes.h>
#endif

#include <errno.h>
#include <pthread.h>
#include <unistd.h>
#include <wchar.h>

#define ERROR_SUCCESS               0L
#define ERROR_INVALID_HANDLE        EINVAL  // 6L
#define ERROR_NOT_ENOUGH_MEMORY     ENOMEM  // 8L
#define ERROR_INVALID_DATA          EINVAL  // 13L
#define ERROR_GEN_FAILURE           EIO     // 31L
#define ERROR_INVALID_PARAMETER     EINVAL  // 87L
#define ERROR_INSUFFICIENT_BUFFER   E2BIG   // 122L

#ifndef WINAPI
#if defined __GNUC__ && (__GNUC__ >= 4 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 3))
#define WINAPI  __attribute__ ((visibility("default")))
#else
#define WINAPI
#endif
#endif

#ifndef IN
#define IN
#endif

#ifndef OUT
#define OUT
#endif

#ifndef FALSE
#define FALSE   0
#endif

#ifndef TRUE
#define TRUE    1
#endif

#ifndef CONST
#define CONST   const
#endif

#ifndef VOID
#define VOID    void
#endif

#define INFINITE                0xFFFFFFFF
#define INVALID_HANDLE_VALUE    -1

#define GetLastError()              ((DWORD) errno)
#define SetLastError(dwErrCode)     errno = (int) dwErrCode
#define Sleep(ms)                   usleep(ms * 1000)

#define InitializeCriticalSection(pCS) {                        \
    pthread_mutexattr_t attr;                                   \
    pthread_mutexattr_init(&attr);                              \
    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);  \
    pthread_mutex_init(pCS, &attr);                             \
    pthread_mutexattr_destroy(&attr);                           \
}
#define DeleteCriticalSection(pCS)      pthread_mutex_destroy(pCS)
#define EnterCriticalSection(pCS)       pthread_mutex_lock(pCS)
#define LeaveCriticalSection(pCS)       pthread_mutex_unlock(pCS)
typedef pthread_mutex_t CRITICAL_SECTION;

typedef VOID *PVOID, *LPVOID;
typedef CONST VOID *LPCVOID;

typedef short SHORT;
typedef long LONG;

typedef unsigned char UCHAR, *PUCHAR;
typedef unsigned short USHORT, *PUSHORT;
typedef unsigned int UINT, *PUINT;
#ifdef __APPLE__
typedef UInt32 ULONG, *PULONG;
#else
typedef unsigned long ULONG, *PULONG;
#endif

typedef unsigned char BYTE, *PBYTE, *LPBYTE;
typedef unsigned short WORD, *PWORD, *LPWORD;
typedef unsigned long DWORD, *PDWORD, *LPDWORD;

typedef int BOOL;
typedef UCHAR BOOLEAN;

typedef char CHAR, *LPSTR;
typedef wchar_t WCHAR, *LPWSTR;

typedef CONST CHAR *LPCSTR;
typedef CONST WCHAR *LPCWSTR;

#ifdef UNICODE
typedef WCHAR TCHAR, *PTCHAR, *LPTSTR;
typedef CONST WCHAR *LPCTSTR;
#define TEXT(quote)     L##quote
#else
typedef CHAR TCHAR, *PTCHAR, *LPTSTR;
typedef CONST CHAR *LPCTSTR;
#define TEXT(quote)     quote
#endif

typedef LPVOID HANDLE, *LPHANDLE;

#endif
